#!/usr/bin/perl -w
#
# identd for MacOS X
#
# Version 1.0 (C) 11.2004 by Beat Rubischon <rubischon@phys.ethz.ch>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

use strict;
use Socket;

$|=1;

my $addr;

# -------------
# get remote IP
# -------------
if (my $peer=getpeername(STDIN)) {
    my ($port, $iaddr)=sockaddr_in($peer);
    $addr=inet_ntoa($iaddr);
} else {
    print STDERR "This script must be run out of inetd.\n";
    exit 1;
}

# ---------
# main loop
# ---------
while(<>) {
    my $src=0;
    my $dst=0;
    my $sts="ERROR";
    my $ret="UNKNOWN-ERROR";

    s/[\n\r]//g;

    if (/^([0-9]+)[ ]*,[ ]*([0-9]+)$/) {
	$src=$1; $dst=$2;
	if ($src>=1 and $src <=65534 and
	    $dst>=1 and $dst <=65534) {

	    if (&checkport($addr, $src, $dst)) {
	        if (my $user=&dolsof($addr, $src, $dst)) {
		    $sts="USERID";
		    $ret="UNIX :$user";
	        }
	    } else {
		$ret="INVALID-PORT";
	    }
	}
    }

    print "$src , $dst : $sts : $ret\n";
}

# ----------------------------
# check if port is really open
# ----------------------------
sub checkport {
    my $addr=shift;
    my $src=shift;
    my $dst=shift;

    my $ok=0;

    open NETSTAT, "/usr/sbin/netstat -tn|" || die "exec failed";

    while(<NETSTAT>) {
	if (m/^tcp4[ ]*[^ ]*[ ]*[^ ]*[ ]*[^ ]*\.$src[ ]*$addr\.$dst[ ]*/) {
	    $ok=1;
	}
    }

    close NETSTAT;

    return $ok;
}

# --------
# run lsof
# --------
sub dolsof {
    my $addr=shift;
    my $src=shift;
    my $dst=shift;

    my $user="NO-USER";

    my $pid=open LSOF, "-|";

    if ($pid == 0) {
	exec "/usr/sbin/lsof", "-i", "tcp:$src", "-n"  || die "exec failed";
    }

    while (<LSOF>) {
	chomp;

	if (m/^[^ ]*[ ]*[0-9]*[ ]*([^ ]*)[ ]*[^ ]*[ ]*IPv4[ ]*[^ ]*[ ]*[^ ]*[ ]*TCP[ ]*[^:]*:([^-]*)->$addr:([^ ]*)[ ]*/) {
	    my $tmpuser=$1;
	    my $srcsrv=$2;
	    my $dstsrv=$3;

	    if (! ($srcsrv=~m/^[0-9]*$/)) {
		my ($name,$aliases,$port,$proto)=getservbyname $srcsrv, "tcp";
		$srcsrv=$port;
	    }

	    if (! ($dstsrv=~m/^[0-9]*$/)) {
		my ($name,$aliases,$port,$proto)=getservbyname $dstsrv, "tcp";
		$dstsrv=$port;
	    }

	    if ($srcsrv == $src and $dstsrv == $dst) {
		$user=$tmpuser;
	    }
	}
    }

    close LSOF;

    return $user;
}
